/****************************************************
*  IrDA-LCD
*  IrDA Ver1.0 115.2Kbps
*  Shigeru Mitsugi
****************************************************/
#include <p18f2550.h>
#include <usart.h>
#include <delays.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

/*** Configuration *******/
#pragma config FOSC = HSPLL_HS
#pragma config WDT = OFF
#pragma config PLLDIV = 5
#pragma config CPUDIV = OSC1_PLL2
#pragma config USBDIV = 2   // Clock source from 96MHz PLL/2
#pragma config PWRT = ON
#pragma config BOR = ON
#pragma config BORV = 1 	// 1:4.33V(Typ)
#pragma config LVP = OFF
#pragma config VREGEN = ON  // USB Voltage Regulator
#pragma config MCLRE = ON
#pragma config PBADEN = OFF

#define LED_G LATBbits.LATB2
#define IrDA_TX LATBbits.LATB3

#pragma udata access accessram
near unsigned char StartBit, StopBit;   // IrDA Bit Data
near unsigned char IrDA_Bit[8];     // IrDA Bit Data

#pragma udata

#define bitcopy(in,out) {if(in) (out)=1; if(!(in)) (out)=0;}

// IrDA Ver1.0Ki 8.68uS/bit, LED ON=1.6uS
//  8.67uS/bit, 1.67uS
#define WriteIrDA_Bit(a) {\
    bitcopy((a & 0x01),IrDA_TX);\
    Delay10TCYx(1);\
    Delay1TCY();\
    Delay1TCY();\
    Delay1TCY();\
    Delay1TCY();\
    Delay1TCY();\
    Delay1TCY();\
    Delay1TCY();\
    Delay1TCY();\
    bitcopy((a & 0x02),IrDA_TX);\
    Delay10TCYx(1);\
    Delay1TCY();\
    Delay1TCY();\
    Delay1TCY();\
    Delay1TCY();\
    Delay1TCY();\
    Delay1TCY();\
    Delay1TCY();\
    bitcopy((a & 0x04),IrDA_TX);\
    Delay10TCYx(1);\
    Delay1TCY();\
    Delay1TCY();\
    Delay1TCY();\
    Delay1TCY();\
    bitcopy((a & 0x08),IrDA_TX);\
    Delay10TCYx(1);\
    Delay1TCY();\
    Delay1TCY();\
    Delay1TCY();\
    Delay1TCY();\
    Delay1TCY();\
    Delay1TCY();\
    Delay1TCY();\
    bitcopy((a & 0x10),IrDA_TX);\
    Delay10TCYx(1);\
    Delay1TCY();\
    Delay1TCY();\
    Delay1TCY();\
    Delay1TCY();\
    Delay1TCY();\
    Delay1TCY();\
    Delay1TCY();\
    Delay1TCY();\
}

void WriteIrDA(char SentData) {
    int i;

    for(i=0; i<8; i++) {
        if ((SentData & 1) == 1) {
            IrDA_Bit[i] = 0;
        }
        else {
            IrDA_Bit[i] = 0x4;
        }

        SentData >>= 1;     
    }

    WriteIrDA_Bit(StartBit);
    WriteIrDA_Bit(IrDA_Bit[0]);
    WriteIrDA_Bit(IrDA_Bit[1]);
    WriteIrDA_Bit(IrDA_Bit[2]);
    WriteIrDA_Bit(IrDA_Bit[3]);
    WriteIrDA_Bit(IrDA_Bit[4]);
    WriteIrDA_Bit(IrDA_Bit[5]);
    WriteIrDA_Bit(IrDA_Bit[6]);
    WriteIrDA_Bit(IrDA_Bit[7]);
    WriteIrDA_Bit(StopBit);
}

void PutsIrDA(char *Command) {
    int i;

    WriteIrDA(Command[0]);  // ŏ1
//  WriteUSART(Command[0]); // for debug

    LED_G = 0;          // Green LED _
    Delay10KTCYx(100);  // wait 83.3mS
    LED_G = 1;          // Green LED 

    for(i=1; i<strlen(Command); i++) {
        WriteIrDA(Command[i]);
//      WriteUSART(Command[i]); // for debug
//      Delay100TCYx(1);            // wait 8.41uS for debug
    }

    WriteIrDA(0);       // f~^=NULL
//  WriteUSART(0x0D);   // CR for debug
//  WriteUSART(0x0A);   // LF for debug
}

// C֐
void main(void) {
    int i, Di, Df, Xpos, Ypos, Xnext, Ynext;
    char TxData;
    char Command[40] = ""; 
    float fWaveData;

    static char sClearLCD[] = "ClearLCD()";
    static char sPrintTitle[] = "Print(010,110,IrDA-LCD Test)";

    static char sPrintMinus[] = "Print(010,000,-1)";
    static char sPrintZero[] = "Print(010,040, 0)";
    static char sPrintPlus[] = "Print(010,080, 1)";
    static char sLineX[] = "Line(030,045,105,045,BLACK)";
    static char sLineY[] = "Line(030,002,030,093,BLACK)";

/*  static char sPixelBL[] = "Pixel(004,000,BLACK,010,011,010,013,010,015,010,017)";
    static char sPixelR[] = "Pixel(004,000,RED,020,021,020,023,020,025,020,027)";
    static char sPixelG[] = "Pixel(004,000,GREEN,010,011,010,013,010,015,010,017)";
    static char sPixelB[] = "Pixel(004,000,BLUE,020,021,020,023,020,025,020,027)";

    static char sLineBL[] = "Line(010,040,110,040,BLACK)";
    static char sLineR[] = "Line(010,040,110,060,RED)";
    static char sLineG[] = "Line(010,040,110,080,GREEN)";
    static char sLineB[] = "Line(010,040,110,100,BLUE)";*/

    TRISB = 0xF0;   // |[gB bit3-0 o
    LATB = 0;   // IrDA_TX = L, Green LED _

    StartBit = 0x4;
    StopBit = 0;

    // USARTݒ 񓯊115.2kbps
/*    OpenUSART(USART_TX_INT_OFF & USART_RX_INT_OFF &
          USART_ASYNCH_MODE & USART_EIGHT_BIT &
          USART_CONT_RX & USART_BRGH_HIGH, 25);

    // Send 'A' for debug
    while(1) {
        WriteIrDA(0x41);
    }*/

    /*  C[v */
    while(1) {
    // R}hM
        // Draw Wave
        PutsIrDA(sClearLCD);
        PutsIrDA(sPrintTitle);
        PutsIrDA(sPrintMinus);
        PutsIrDA(sPrintZero);
        PutsIrDA(sPrintPlus);
        PutsIrDA(sLineX);
        PutsIrDA(sLineY);

        // Draw sin Wave
        for (i=0; i<=360; i=i+15) {
            fWaveData = sin((float)i *3.1416 / 180);
            fWaveData = (fWaveData+1) *40 +5;   // 5`85
            Di = (int)fWaveData;
                Df = (int)((fWaveData - (int)fWaveData) *100);
                sprintf(Command,"Print(010,095,%02d.%02d)",Di,Df);  // %f(_)͖T|[g
            PutsIrDA(Command);

            Xnext = i/5 +30;
            Ynext = Di;
            if (i != 0) {
                    sprintf(Command,"Line(%03d,%03d,%03d,%03d,RED)", Xpos, Ypos, Xnext, Ynext);
                PutsIrDA(Command);
            }
            Xpos = Xnext;
            Ypos = Ynext;
        }

        // Draw cos Wave
        for (i=0; i<=360; i=i+15) {
            fWaveData = cos((float)i *3.1416 / 180);
            fWaveData = (fWaveData+1) *40 +5;
            Di = (int)fWaveData;
                Df = (int)((fWaveData - (int)fWaveData) *100);
                sprintf(Command,"Print(010,095,%02d.%02d)",Di,Df);
            PutsIrDA(Command);

            Xnext = i/5 +30;
            Ynext = Di;
            if (i != 0) {
                    sprintf(Command,"Line(%03d,%03d,%03d,%03d,BLUE)", Xpos, Ypos, Xnext, Ynext);
                PutsIrDA(Command);
            }
            Xpos = Xnext;
            Ypos = Ynext;
        }

        // Draw Line
/*      PutsIrDA(sClearLCD);
        PutsIrDA(sPrintTitle);

        PutsIrDA(sPixelG);
        PutsIrDA(sPixelB);

        PutsIrDA(sLineBL);
        PutsIrDA(sLineR);
        PutsIrDA(sLineG);
        PutsIrDA(sLineB);*/

        // 232C to IrDA
/*      while(!DataRdyUSART()); // M҂
        TxData = ReadUSART();       // 1M
        WriteUSART(TxData);     // ܂ԂM
        if (TxData == 0x0D) {       // 0x0D:CR
            WriteIrDA(0);           // 0:NULL
        }
        else {
            WriteIrDA(TxData);
        }*/
    }
}